@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: June 24, 2017
+# Last Change: July 1, 2017
 # URL: https://humanfriendly.readthedocs.io
 
 """
@@ -24,6 +24,7 @@ I have several reasons for doing so:
 """
 
 # Standard library module
+import functools
 import logging
 import os
 import pipes
@@ -31,15 +32,17 @@ import shutil
 import sys
 import tempfile
 import time
-import unittest
 
 # Modules included in our package.
-from humanfriendly.compat import StringIO
+from humanfriendly.compat import StringIO, unicode, unittest
 from humanfriendly.text import compact, random_string
 
 # Initialize a logger for this module.
 logger = logging.getLogger(__name__)
 
+# A unique object reference used to detect missing attributes.
+NOTHING = object()
+
 # Public identifiers that require documentation.
 __all__ = (
     'CallableTimedOut',
@@ -52,6 +55,7 @@ __all__ = (
     'TemporaryDirectory',
     'TestCase',
     'retry',
+    'run_cli',
 )
 
 
@@ -93,6 +97,45 @@ def retry(func, timeout=60, exc_type=AssertionError):
             pause *= 2
 
 
+def run_cli(entry_point, *arguments, **options):
+    """
+    Test a command line entry point.
+
+    :param entry_point: The function that implements the command line interface
+                        (a callable).
+    :param arguments: Any positional arguments (strings) become the command
+                      line arguments (:data:`sys.argv` items 1-N).
+    :param options: The following keyword arguments are supported:
+
+                    **input**
+                     Refer to :class:`CaptureOutput`.
+                    **merged**
+                     Refer to :class:`CaptureOutput`.
+                    **program_name**
+                     Used to set :data:`sys.argv` item 0.
+    :returns: A tuple with two values:
+
+              1. The return code (an integer).
+              2. The captured output (a string).
+    """
+    program_name = options.pop('program_name', sys.executable)
+    # Start capturing the standard output and error streams.
+    with CaptureOutput(**options) as output:
+        # Temporarily override sys.argv.
+        arguments = list(arguments)
+        arguments.insert(0, program_name)
+        with PatchedAttribute(sys, 'argv', arguments):
+            try:
+                # Call the command line interface.
+                entry_point()
+            except SystemExit as e:
+                # Intercept sys.exit() calls.
+                returncode = e.code
+            else:
+                returncode = 0
+            return returncode, output.getvalue()
+
+
 class CallableTimedOut(Exception):
 
     """Raised by :func:`retry()` when the timeout expires."""
@@ -125,7 +168,7 @@ class PatchedAttribute(ContextManager):
         self.object_to_patch = obj
         self.attribute_to_patch = name
         self.patched_value = value
-        self.original_value = None
+        self.original_value = NOTHING
 
     def __enter__(self):
         """
@@ -136,7 +179,7 @@ class PatchedAttribute(ContextManager):
         # Enable composition of context managers.
         super(PatchedAttribute, self).__enter__()
         # Patch the object's attribute.
-        self.original_value = getattr(self.object_to_patch, self.attribute_to_patch)
+        self.original_value = getattr(self.object_to_patch, self.attribute_to_patch, NOTHING)
         setattr(self.object_to_patch, self.attribute_to_patch, self.patched_value)
         return self.object_to_patch
 
@@ -145,7 +188,10 @@ class PatchedAttribute(ContextManager):
         # Enable composition of context managers.
         super(PatchedAttribute, self).__exit__(exc_type, exc_value, traceback)
         # Restore the object's attribute.
-        setattr(self.object_to_patch, self.attribute_to_patch, self.original_value)
+        if self.original_value is NOTHING:
+            delattr(self.object_to_patch, self.attribute_to_patch)
+        else:
+            setattr(self.object_to_patch, self.attribute_to_patch, self.original_value)
 
 
 class PatchedItem(ContextManager):
@@ -163,7 +209,7 @@ class PatchedItem(ContextManager):
         self.object_to_patch = obj
         self.item_to_patch = item
         self.patched_value = value
-        self.original_value = None
+        self.original_value = NOTHING
 
     def __enter__(self):
         """
@@ -174,7 +220,10 @@ class PatchedItem(ContextManager):
         # Enable composition of context managers.
         super(PatchedItem, self).__enter__()
         # Patch the object's item.
-        self.original_value = self.object_to_patch[self.item_to_patch]
+        try:
+            self.original_value = self.object_to_patch[self.item_to_patch]
+        except KeyError:
+            self.original_value = NOTHING
         self.object_to_patch[self.item_to_patch] = self.patched_value
         return self.object_to_patch
 
@@ -183,7 +232,10 @@ class PatchedItem(ContextManager):
         # Enable composition of context managers.
         super(PatchedItem, self).__exit__(exc_type, exc_value, traceback)
         # Restore the object's item.
-        self.object_to_patch[self.item_to_patch] = self.original_value
+        if self.original_value is NOTHING:
+            del self.object_to_patch[self.item_to_patch]
+        else:
+            self.object_to_patch[self.item_to_patch] = self.original_value
 
 
 class TemporaryDirectory(ContextManager):
@@ -344,17 +396,25 @@ class CaptureOutput(ContextManager):
 
     """Context manager that captures what's written to :data:`sys.stdout` and :data:`sys.stderr`."""
 
-    def __init__(self, merged=False):
+    def __init__(self, merged=False, input=''):
         """
         Initialize a :class:`CaptureOutput` object.
 
         :param merged: :data:`True` to merge the streams,
                        :data:`False` to capture them separately.
+        :param input: The data that reads from :data:`sys.stdin`
+                      should return (a string).
         """
+        self.stdin = StringIO(input)
         self.stdout = StringIO()
         self.stderr = self.stdout if merged else StringIO()
-        self.stdout_context = PatchedAttribute(sys, 'stdout', self.stdout)
-        self.stderr_context = PatchedAttribute(sys, 'stderr', self.stderr)
+        self.patched_attributes = [
+            PatchedAttribute(sys, name, getattr(self, name))
+            for name in ('stdin', 'stdout', 'stderr')
+        ]
+
+    stdin = None
+    """The :class:`~humanfriendly.compat.StringIO` object used to feed the standard input stream."""
 
     stdout = None
     """The :class:`~humanfriendly.compat.StringIO` object used to capture the standard output stream."""
@@ -365,15 +425,15 @@ class CaptureOutput(ContextManager):
     def __enter__(self):
         """Start capturing what's written to :data:`sys.stdout` and :data:`sys.stderr`."""
         super(CaptureOutput, self).__enter__()
-        self.stdout_context.__enter__()
-        self.stderr_context.__enter__()
+        for context in self.patched_attributes:
+            context.__enter__()
         return self
 
     def __exit__(self, exc_type=None, exc_value=None, traceback=None):
         """Stop capturing what's written to :data:`sys.stdout` and :data:`sys.stderr`."""
         super(CaptureOutput, self).__exit__(exc_type, exc_value, traceback)
-        sys.stdout_context.__exit__(exc_type, exc_value, traceback)
-        sys.stderr_context.__exit__(exc_type, exc_value, traceback)
+        for context in self.patched_attributes:
+            context.__exit__(exc_type, exc_value, traceback)
 
     def getvalue(self):
         """Get the text written to :data:`sys.stdout`."""
@@ -384,6 +444,40 @@ class TestCase(unittest.TestCase):
 
     """Subclass of :class:`unittest.TestCase` with automatic logging and other miscellaneous features."""
 
+    exceptionsToSkip = []
+    """A list of exception types that are translated into skipped tests."""
+
+    def __init__(self, *args, **kw):
+        """Wrap test methods using :func:`skipTestWrapper()`."""
+        # Initialize our superclass.
+        super(TestCase, self).__init__(*args, **kw)
+        # Wrap all of the test methods so that we can customize the
+        # skipping of tests based on the exceptions they raise.
+        for name in dir(self.__class__):
+            if name.startswith('test_'):
+                setattr(self, name, functools.partial(
+                    self.skipTestWrapper,
+                    getattr(self, name),
+                ))
+
+    def assertRaises(self, exception, callable, *args, **kwds):
+        """
+        Replacement for :func:`unittest.TestCase.assertRaises()` that returns the exception.
+
+        Refer to the :func:`unittest.TestCase.assertRaises()` documentation for
+        details on argument handling. The return value is the caught exception.
+
+        .. warning:: This method does not support use as a context manager.
+        """
+        try:
+            callable(*args, **kwds)
+        except exception as e:
+            # Return the expected exception as a regular return value.
+            return e
+        else:
+            # Raise an exception when no exception was raised :-).
+            assert False, "Expected an exception to be raised!"
+
     def setUp(self, log_level=logging.DEBUG):
         """setUp(log_level=logging.DEBUG)
         Automatically configure logging to the terminal.
@@ -419,17 +513,70 @@ class TestCase(unittest.TestCase):
         # logging output that the test method is likely going to generate.
         sys.stderr.write("\n")
 
+    def shouldSkipTest(self, exception):
+        """
+        Decide whether a test that raised an exception should be skipped.
+
+        :param exception: The exception that was raised by the test.
+        :returns: :data:`True` to translate the exception into a skipped test,
+                  :data:`False` to propagate the exception as usual.
+
+        The :func:`shouldSkipTest()` method skips exceptions listed in the
+        :attr:`exceptionsToSkip` attribute. This enables subclasses of
+        :class:`TestCase` to customize the default behavior with a one liner.
+        """
+        return isinstance(exception, tuple(self.exceptionsToSkip))
+
     def skipTest(self, text, *args, **kw):
         """
-        Enable backwards compatible "marking of tests to skip".
+        Enable skipping of tests.
 
-        By calling this method from a return statement in the test to be
-        skipped, that test can be marked as skipped when possible, without
-        breaking the test suite when :func:`unittest.TestCase.skipTest()`
-        isn't available (in Python 2.6 and 3.0).
+        This method was added in humanfriendly 3.3 as a fall back for the
+        :func:`unittest.TestCase.skipTest()` method that was added in Python
+        2.7 and 3.1 (because humanfriendly also supports Python 2.6).
+
+        Since then `humanfriendly` has gained a conditional dependency on
+        unittest2_ which enables actual skipping of tests (instead of just
+        mocking it) on Python 2.6.
+
+        This method now remains for backwards compatibility (and just because
+        it's a nice shortcut).
+
+        .. _unittest2: https://pypi.python.org/pypi/unittest2
+        """
+        raise unittest.SkipTest(compact(text, *args, **kw))
+
+    def skipTestWrapper(self, test_method, *args, **kw):
+        """
+        Wrap test methods to translate exceptions into skipped tests.
+
+        :param test_method: The test method to wrap.
+        :param args: The positional arguments to the test method.
+        :param kw: The keyword arguments to the test method.
+        :returns: The return value of the test method.
+
+        When a :class:`TestCase` object is initialized, :func:`__init__()`
+        wraps all of the ``test_*`` methods with :func:`skipTestWrapper()`.
+
+        When a test method raises an exception, :func:`skipTestWrapper()` will
+        catch the exception and call :func:`shouldSkipTest()` to decide whether
+        to translate the exception into a skipped test.
+
+        When :func:`shouldSkipTest()` returns :data:`True` the exception is
+        swallowed and :exc:`unittest.SkipTest` is raised instead of the
+        original exception.
         """
-        reason = compact(text, *args, **kw)
         try:
-            super(TestCase, self).skipTest(reason)
-        except AttributeError:
-            logger.warning("%s", reason)
+            return test_method(*args, **kw)
+        except BaseException as e:
+            if self.shouldSkipTest(e):
+                if isinstance(e, unittest.SkipTest):
+                    # We prefer to preserve the original
+                    # exception and stack trace.
+                    raise
+                else:
+                    # If the original exception wasn't a unittest.SkipTest
+                    # exception then we will translate it into one.
+                    raise unittest.SkipTest(unicode(e))
+            else:
+                raise
